#!/usr/bin/env python3

# FIXME: Ideally byovm will learn to import and execute directly
# -- vila 2018-02-17

# This file is part of Build Your Own CI
#
# Copyright 2018 Vincent Ladeuil
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

# /!\ This is expanded by byovm, caveat emptor. {} is not expanded but it's a
# close call => FIXME: import and execute

import logging
import os
import sys

import byov


logger = logging.getLogger(__name__)
host_def_path = '{{byoci}.host.definition}'
sys.path.insert(0, host_def_path)


# FIXME: Come on I just set sys.path above so the following import can succeed,
# surely pep8 shouldn't complain ? -- vila 2018-02-17
from byoci.host import (  # noqa
    config,
    commands,
    options,
)


def upload_master_ssh_keys():
    # Install ssh keys on the master
    secrets = commands.SetupSecrets('{byoci}')
    name, path = secrets.master_key()
    logger.debug('Setting master ssh keys: {} at {}.'.format(name, path))
    commands.push_ssh_file('{vm.name}', name, path)
    commands.push_ssh_file('{vm.name}', name + '.pub', path + '.pub')


def upload_expanded_templates():
    master_name = '{vm.name}'
    master_conf = config.VmStack(master_name)
    # Make target dirs
    bin_rpath = os.path.join(master_conf.get('jenkins.home'), 'bin')
    commands.run_from_hook(['shell', master_name, 'mkdir', '-p', bin_rpath])
    expanded_rpath = os.path.join(master_conf.get('jenkins.home'), 'expanded')
    commands.run_from_hook(['shell', master_name, 'mkdir', '-p',
                            expanded_rpath])
    # Upload jenkins templates
    tmpl_path = os.path.join(host_def_path, 'setup', 'templates', 'jenkins')
    for f in os.listdir(tmpl_path):
        lpath = os.path.join(tmpl_path, f)
        rpath = os.path.join(expanded_rpath, f)
        logger.debug('Uploading template: {} at {}.'.format(lpath, rpath))
        commands.run_from_hook(['push', master_name, '@' + lpath, rpath])
    # The slave connection script
    start_slave = 'start-slave-via-ssh'
    lpath = os.path.join(host_def_path, 'setup', 'templates', start_slave)
    rpath = os.path.join(bin_rpath, start_slave)
    logger.debug('Uploading script: {} at {}.'.format(lpath, rpath))
    commands.run_from_hook(['push', master_name,
                            '@' + lpath,
                            rpath])
    # The ssh files
    ssh_path = os.path.join(host_def_path, 'setup', 'templates', 'ssh')
    commands.push_ssh_file(master_name, 'config',
                           os.path.join(ssh_path, 'master_config'))


if __name__ == '__main__':
    os.environ['BYOV_PATH'] = ':'.join(byov.path)
    commands.setup_logging('{logging.level}'.upper())
    upload_master_ssh_keys()
    upload_expanded_templates()
