#!/usr/bin/env python3

# FIXME: Ideally byov will learn to import and execute directly
# -- vila 2018-02-17

# This file is part of Build Your Own CI
#
# Copyright 2018 Vincent Ladeuil
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

# /!\ This is expanded by byov, caveat emptor

import logging
import os
import sys
import tempfile


import byov

from byov import subprocesses


logger = logging.getLogger(__name__)
sys.path.insert(0, '{{byoci}.host.definition}')


# FIXME: Come on... I just set sys.path above so the following import can
# succeed, surely pep8 shouldn't complain ? -- vila 2018-02-17
from byoci.host import (  # noqa
    config,
    commands,
    options,
)


def master_knows_slave():
    logger.debug('Telling master about {vm.name}')
    master_name = '{{byoci}.master}'
    mconf = config.VmStack(master_name)
    commands.run_from_hook(
        ['ssh-register', master_name, '{vm.name}'])
    sconf = config.VmStack('{vm.name}')
    # Add the proxy in master known_hosts if needed.
    auth_proxy = sconf.get('ssh.proxy.known_host')
    if auth_proxy:
        # When using the proxy, it must be a known host
        commands.run_from_hook(
            ['shell', master_name,
             'echo', auth_proxy, '>>', '~/.ssh/known_hosts'])
    # Create slave config for ssh access
    ssh_conf_path = os.path.join('setup', 'templates', 'ssh', 'slave_config')
    found_path = subprocesses.which(ssh_conf_path, byov.path, mode=os.F_OK)

    tmp = tempfile.NamedTemporaryFile(delete=False, mode='w')
    with open(found_path) as f:
        content = f.read()
        expanded = sconf.expand_options(content)
        tmp.write(expanded)
    proxy_cmd = sconf.get('ssh.proxy.command')
    if proxy_cmd:
        tmp.write('\tProxyCommand {}"\n'.format(proxy_cmd))
    tmp.close()
    logger.debug('tmp.name: {}'.format(tmp.name))
    try:
        dst = os.path.join(mconf.get('jenkins.home'),
                           '.ssh', '{vm.name}.config')
        commands.run_from_hook(['push', master_name, tmp.name, dst])
    finally:
        os.remove(tmp.name)


def upload_localhost_config():
    # Upload a config file for the slave so it can expand scripts
    logger.debug('Uploading the localhost config')
    option_set = commands.matching_options(
        '{vm.name}', ['^byoci$', '^{byoci}\.', '^launchpad\.', '^gitlab\.',
                      '^jenkins\.'])
    tmp = tempfile.NamedTemporaryFile(delete=False, mode='w')
    tmp.write('[localhost]\n')
    for name in sorted(option_set.keys()):
        tmp.write('{} = {}\n'.format(name, option_set[name]))
    tmp.close()
    logger.debug('tmp.name: {}'.format(tmp.name))
    localhost_dir = os.path.join('~', '.config', 'byov', 'conf.d')
    localhost_path = os.path.join(localhost_dir, 'byov-localhost.conf')
    try:
        commands.run_from_hook(
            ['shell', '{vm.name}', 'mkdir', '-p', localhost_dir])
        commands.run_from_hook(['push', '{vm.name}', tmp.name, localhost_path])
    finally:
        os.remove(tmp.name)


if __name__ == '__main__':
    commands.setup_logging('{logging.level}'.upper())
    os.environ['BYOV_PATH'] = ':'.join(byov.path)
    secrets = commands.SetupSecrets('{byoci}')
    # Add master key to slave authorized_keys
    logger.debug('Allowing master on {vm.name}')
    name, path = secrets.master_key()
    commands.run_from_hook(
        ['ssh-authorize', '{vm.name}', path + '.pub'])
    # Add slave ssh key to master known_hosts
    master_knows_slave()
    # Install slave key for launchpad write access
    name, path = secrets.slave_key()
    commands.push_ssh_file('{vm.name}', name, path)
    commands.push_ssh_file('{vm.name}', name + '.pub', path + '.pub')
    upload_localhost_config()
