// http://javadoc.jenkins-ci.org/
import java.io.FileInputStream
import hudson.model.Computer
import hudson.model.Item
import hudson.model.User;
import hudson.security.GlobalMatrixAuthorizationStrategy
import hudson.security.Permission
import hudson.slaves.OfflineCause
import jenkins.model.Jenkins
import jenkins.model.JenkinsLocationConfiguration
import org.jenkinsci.plugins.matrixauth.AuthorizationType;
import org.jenkinsci.plugins.matrixauth.PermissionEntry;

def jenk = jenkins.model.Jenkins.getInstance();

def security = jenk.getSecurityRealm()

def strategy = new hudson.security.GlobalMatrixAuthorizationStrategy()


// The initial admin user is created with a default password. We use the
// same for all users. Users should be taught to change those passwords
// asap.
def pass_path = '/var/lib/jenkins/secrets/initialAdminPassword'
def default_password = new FileInputStream(pass_path).readLines()[0]

// Some users need to exist, create them with default password if they
// don't. Users should be taught to change those passwords asap.
for (username in '{{byoci}.admin.users}'.split(', +')){
    User user = hudson.model.User.getById(username, false);
    if (null == user) {
        security.createAccount(username, default_password)
    }
}

// Anonymous users can read
anon = PermissionEntry.user('anonymous')
strategy.add(jenkins.model.Jenkins.READ, anon)
strategy.add(hudson.model.Item.READ, anon)

// Authenticated users can read jobs
authed = PermissionEntry.group('authenticated')
strategy.add(jenkins.model.Jenkins.READ, authed)
strategy.add(hudson.model.Item.READ, authed)

// Admins administer
for (user in '{{byoci}.admin.users}'.split(', +')){
    pentry = PermissionEntry.user(user)
    strategy.add(jenkins.model.Jenkins.ADMINISTER, pentry)
}
// The monitor is an admin too (but see FIXME in
// ../../../byoci/monitor/commands.py
pentry = PermissionEntry.user('{{byoci}.monitor.user}')
strategy.add(jenkins.model.Jenkins.ADMINISTER, pentry)

jenk.setAuthorizationStrategy(strategy)

// Disable job execution on the master
def why = new hudson.slaves.OfflineCause.ByCLI(
    'No job execution allowed on master')
jenk.toComputer().setTemporarilyOffline(true, why)
jenk.setNumExecutors(0)

// Setup location: url and email
def public_url = '{{byoci}.master.url}'
if (public_url != 'default') {
    jlc = jenkins.model.JenkinsLocationConfiguration.get()
    jlc.setUrl(public_url) 
    jlc.setAdminAddress('{{byoci}.admin.email}') 
    jlc.save()
}

// Now, get rid of 'admin' created by jenkins during bootstrap for no good
// reasons

admin = User.get('admin')
admin.delete()

jenk.save()
