// First of all, matrix-auth and openid (and their dependencies) need to be
// installed to provide GlobalMatrixAuthorizationStrategy which is a
// requirement to define access rights (see access.groovy). openid itself is
// required so sso can be configured and used.

import jenkins.model.Jenkins

def jenk = Jenkins.getInstance();

// The required plugins are hard-coded as it makes little sense to allow
// user to remove some (and adding more will probably require associated
// changes).

// cloudbees-folder needs to be installed or reloading existing jobs fails
def plugins = ['matrix-auth', 'cloudbees-folder']

jenk.updateCenter.updateAllSites()
for (pname in plugins) {
    plugin = jenk.updateCenter.getPlugin(pname)
    if (plugin == null) {
        println(pname + ' is not a known plugin')
        System.exit(1)
    }
    for (dep in plugin.getNeededDependencies()) {
        dep.doInstallNow()
    }
    plugin.doInstallNow()
}

// Wait for plugins to be installed (i.e. seen in pluginManager.plugins)
def time_to_wait = 120000 // Two minutes
def keep_waiting = time_to_wait
def nap_duration = 2000 // sleep for 2 seconds
def  done = false
while (!done &&  keep_waiting) {

    installed = jenk.pluginManager.plugins*.getShortName()
    all_found = true
    for (pname in plugins) {
        if ( !(pname in installed)) {
            println(pname + ' is not installed yet, keep waiting for '
                    + keep_waiting + ' seconds')
            all_found = false
        }
    }
    if (all_found) {
        done = true
    }
    else {
        keep_waiting = keep_waiting - nap_duration
        sleep(nap_duration)
    }
}
if (done) {
    println(plugins + ' are installed')
}
else {
    println(plugins + ' are not installed after '
            + (time_to_wait / 1000) + 'seconds')
    System.exit(1)
}

def plugins_installed_file = new File('{jenkins.home}/.plugins-installed')
plugins_installed_file.write("$plugins")
